// ==============================================================
//LongShot.cpp
//Class description for LongShot vessel class
//
//Addon author: Jorge Iglesias (AKA Hispa)
//Orbiter (C) 2002-2010 Martin Schweiger
//All rights reserved
//
//This file was generated by Xantcha's Orbiter vessel template
// ==============================================================

#include "LongShot.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double MASS = 35824560;
const double FUELMASS = 98654240;
const double ISP = 80000;
const double MAXMAINTH = 5e7;
const double MAXHOVERTH = 0;

const double maximo=2e5;
const double minimo=1e4;

double MAXRCSTH = maximo;


//Caractersticas de las partculas de humo
		static PARTICLESTREAMSPEC exhaust_main = {
		0, 2.0, 20, 150, 0.1, 0.2, 16, 2.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};

////////////////////////////////////////////////////////
// SendHudMessage
////////////////////////////////////////////////////////
char *LongShot::SendHudMessage()
{
dHudMessageDelay=15;
return cUmmuHudDisplay;
}

// ==============================================================
// Overloaded callback functions
// ==============================================================
// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void LongShot::clbkSetClassCaps (FILEHANDLE cfg)
{
Crew.InitUmmu(GetHandle());// Must always be the first one
Crew.DefineAirLockShape(FALSE,-3,3,-4,4,-7,7);// Airlock closed, 6 meter large 8 meter high 14 meter long
Crew.SetMembersPosRotOnEVA(_V(0,0,3),_V(0,0,0));// position when EVA, 3 meter in front of ship
Crew.SetMaxSeatAvailableInShip(50);// Max Numbers of UMMU for this ship
// Add one default member for when the ship is spawned by scenario editor
Crew.AddCrewMember("Heywood Floyd",54,65,82,"Passenger");
iSelectedUmmuMember =0; // Initialize of select member variable

// Valor por defecto del transpondedor
transpondedor=0;
// Activar
EnableTransponder(true);
SetTransponderChannel(transpondedor);

iActiveDockNumber=0;
SetUMMUAirlockPos();

// The HUD display method variables
cUmmuHudDisplay[0] =0;// Initialisation of UMmu hud char variable
dHudMessageDelay =0;// Initialisation of UMmu delay variable
strcpy(SendHudMessage(),"Welcome aboard ! E=EVA 1,2=select UMmu A=Open/Close airlock S=info");

// Localizacin de los motores principales
MainEngineOfs[0]=_V(-8.5,0,-338);
MainEngineOfs[1]=_V(-4.5,0,-338);
MainEngineOfs[2]=_V(4.5,0,-338);
MainEngineOfs[3]=_V(8.5,0,-338);
MainEngineOfs[4]=_V(0,-8.5,-338);
MainEngineOfs[5]=_V(0,-4.5,-338);
MainEngineOfs[6]=_V(0,4.5,-338);
MainEngineOfs[7]=_V(0,8.5,-338);

// Creacin del tanque de combustible
ph_main=CreatePropellantResource(FUELMASS);

	SetSize (545);		//size parameter

		// mass	
	SetEmptyMass (MASS);	
		//aerodynamic properties
	SetCW (0.09, 0.09, 2, 1.4);
	SetCrossSections (_V(26.32,26.34,14.67));
	SetRotDrag (_V(3.93,3.93,3.93));
	SetPitchMomentScale (1e-4);
	SetBankMomentScale (1e-4);		
		//principal moment of inertia
	SetPMI (_V(2.28,2.31,0.79));

	// ***************** thruster definitions *******************

int i;

	//Se crean los motores principales y sus escapes
	for (i=0; i<8; i++)
	{
		th_main[i] = CreateThruster (_V(0,0,0), _V(0,0,1), MAXMAINTH, ph_main, ISP);
		AddExhaust (th_main[i],1,2.60,_V(MainEngineOfs[i].x,MainEngineOfs[i].y,MainEngineOfs[i].z-2),_V(0,0,-1));
		AddExhaustStream (th_main[i], MainEngineOfs[i], &exhaust_main);
	};

	// Se crea el grupo motor principal
	thg_main = CreateThrusterGroup (th_main, 8, THGROUP_MAIN);

	// Impulsores de control de posicin
	// Para mejorar la navegabilidad, los motores estn ubicados en posiciones diferentes a los escapes

	// Grupo delantero derecho

	// Delantero derecho superior
	th_rcs[0]=CreateThruster(_V(20,0,100),_V(0,-1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[0],3,0.3,_V(5.75,1.48,190),_V(1,1,0));
	// Delantero derecho inferior
	th_rcs[1]=CreateThruster(_V(20,0,100),_V(0,1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[1],3,0.3,_V(5.75,-1.48,190),_V(1,-1,0));
	// Delantero derecho proa
	th_rcs[2]=CreateThruster(_V(20,0,100),_V(0,0,-1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[2],3,0.3,_V(5.79,0,192),_V(1,0,1));
	// Delantero derecho popa
	th_rcs[3]=CreateThruster(_V(20,0,100),_V(0,0,1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[3],3,0.3,_V(5.79,0,189),_V(1,0,-1));
	// Delantero derecho exterior
	th_rcs[4]=CreateThruster(_V(20,0,100),_V(-1,0,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[4],3,0.3,_V(6.4,0,190),_V(1,0,0));

	// Grupo delantero izquierdo

	// Delantero izquierdo superior
	th_rcs[5]=CreateThruster(_V(-20,0,100),_V(0,-1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[5],3,0.3,_V(-5.75,1.48,190),_V(-1,1,0));
	// Delantero izquierdo inferior
	th_rcs[6]=CreateThruster(_V(-20,0,100),_V(0,1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[6],3,0.3,_V(-5.75,-1.48,190),_V(-1,-1,0));
	// Delantero izquierdo proa
	th_rcs[7]=CreateThruster(_V(-20,0,100),_V(0,0,-1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[7],3,0.3,_V(-5.79,0,192),_V(-1,0,1));
	// Delantero izquierdo popa
	th_rcs[8]=CreateThruster(_V(-20,0,100),_V(0,0,1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[8],3,0.3,_V(-5.79,0,189),_V(-1,0,-1));
	// Delantero izquierdo exterior
	th_rcs[9]=CreateThruster(_V(-20,0,100),_V(1,0,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[9],3,0.3,_V(-6.4,0,190),_V(-1,0,0));

	// Grupo trasero derecho

	// Trasero derecho superior
	th_rcs[10]=CreateThruster(_V(20,0,-100),_V(0,-1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[10],3,0.3,_V(5.75,1.48,-301),_V(1,1,0));
	// Trasero derecho inferior
	th_rcs[11]=CreateThruster(_V(20,0,-100),_V(0,1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[11],3,0.3,_V(5.75,-1.48,-301),_V(1,-1,0));
	// Trasero derecho proa
	th_rcs[12]=CreateThruster(_V(20,0,-100),_V(0,0,-1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[12],3,0.3,_V(5.79,0,-299),_V(1,0,1));
	// Trasero derecho popa
	th_rcs[13]=CreateThruster(_V(20,0,-100),_V(0,0,1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[13],3,0.3,_V(5.79,0,-302),_V(1,0,-1));
	// trasero derecho exterior
	th_rcs[14]=CreateThruster(_V(20,0,-100),_V(-1,0,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[14],3,0.3,_V(6.4,0,-301),_V(1,0,0));

	// Grupo trasero izquierdo

	// Trasero izquierdo superior
	th_rcs[15]=CreateThruster(_V(-20,0,-100),_V(0,-1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[15],3,0.3,_V(-5.75,1.48,-301),_V(-1,1,0));
	// Trasero izquierdo inferior
	th_rcs[16]=CreateThruster(_V(-20,0,-100),_V(0,1,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[16],3,0.3,_V(-5.75,-1.48,-301),_V(-1,-1,0));
	// Trasero izquierdo proa
	th_rcs[17]=CreateThruster(_V(-20,0,-100),_V(0,0,-1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[17],3,0.3,_V(-5.79,0,-299),_V(-1,0,1));
	// Trasero izquierdo popa
	th_rcs[18]=CreateThruster(_V(-20,0,-100),_V(0,0,1),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[18],3,0.3,_V(-5.79,0,-302),_V(-1,0,-1));
	// Trasero izquierdo exterior
	th_rcs[19]=CreateThruster(_V(-20,0,-100),_V(1,0,0),MAXRCSTH,ph_main,ISP);
	AddExhaust (th_rcs[19],3,0.3,_V(-6.4,0,-301),_V(-1,0,0));

	// Se crean los grupos motores para los diferentes movimientos de la nave

	// 1-Movimientos rotacionales

	// Grupo motor de cabeceo hacia arriba "PITCH-UP"
	th_group[0]=th_rcs[1];
	th_group[1]=th_rcs[6];
	th_group[2]=th_rcs[10];
	th_group[3]=th_rcs[15];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_PITCHUP);

	// Grupo motor de cabeceo hacia abajo "PITCH-DOWN"
	th_group[0]=th_rcs[0];
	th_group[1]=th_rcs[5];
	th_group[2]=th_rcs[11];
	th_group[3]=th_rcs[16];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_PITCHDOWN);

	// Grupo motor de balanceo derecha "YAW-RIGHT"
	th_group[0]=th_rcs[9];
	th_group[1]=th_rcs[14];
	CreateThrusterGroup(th_group,2,THGROUP_ATT_YAWRIGHT);

	// Grupo motor de balanceo izquierda "YAW-LEFT"
	th_group[0]=th_rcs[4];
	th_group[1]=th_rcs[19];
	CreateThrusterGroup(th_group,2,THGROUP_ATT_YAWLEFT);

	// Grupo motor de alabeo derecho "BANK-RIGHT"
	th_group[0]=th_rcs[0];
	th_group[1]=th_rcs[6];
	th_group[2]=th_rcs[10];
	th_group[3]=th_rcs[16];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_BANKRIGHT);

	// Grupo motor de alabeo izquierdo "BANK-LEFT"
	th_group[0]=th_rcs[1];
	th_group[1]=th_rcs[5];
	th_group[2]=th_rcs[11];
	th_group[3]=th_rcs[15];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_BANKLEFT);

	// 2-Movimientos traslacionales

	// Grupo motor de traslacin derecha "RIGHT"
	// Este grupo lo podan formar los impulsores izquierdos delantero de proa y trasero de popa
	th_group[0]=th_rcs[9];
	th_group[1]=th_rcs[19];
	CreateThrusterGroup(th_group,2,THGROUP_ATT_RIGHT);

	// Grupo motor de traslacin izquierda "LEFT"
	// Este grupo lo podan formar los impulsores derechos delantero de proa y trasero de popa
	th_group[0]=th_rcs[4];
	th_group[1]=th_rcs[14];
	CreateThrusterGroup(th_group,2,THGROUP_ATT_LEFT);

	// Grupo motor de traslacin arriba "DOWN"
	th_group[0]=th_rcs[0];
	th_group[1]=th_rcs[5];
	th_group[2]=th_rcs[10];
	th_group[3]=th_rcs[15];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_DOWN);

	// Grupo motor de traslacin abajo "UP"
	th_group[0]=th_rcs[1];
	th_group[1]=th_rcs[6];
	th_group[2]=th_rcs[11];
	th_group[3]=th_rcs[16];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_UP);

	// Grupo motor de traslacin adelante "FORWARD"
	th_group[0]=th_rcs[3];
	th_group[1]=th_rcs[8];
	th_group[2]=th_rcs[13];
	th_group[3]=th_rcs[18];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_FORWARD);

	// Grupo motor de traslacin atrs "BACK"
	th_group[0]=th_rcs[2];
	th_group[1]=th_rcs[7];
	th_group[2]=th_rcs[12];
	th_group[3]=th_rcs[17];
	CreateThrusterGroup(th_group,4,THGROUP_ATT_BACK);

// Puertos de atraque

	CreateDock (_V(0,0,207.2), _V(0,0,1), _V(0,1,0));
	CreateDock (_V(4.57,0,-4.67), _V(1,0,0), _V(0,0,-1));
	CreateDock (_V(-4.57,0,-4.67), _V(-1,0,0), _V(0,0,-1));
	CreateDock (_V(20,0,49), _V(0,0,-1), _V(1,0,0));
	CreateDock (_V(-20,0,49), _V(0,0,-1), _V(-1,0,0));
	CreateDock (_V(0,-20,49), _V(0,0,-1), _V(0,-1,0));
	CreateDock (_V(0,20,49), _V(0,0,-1), _V(0,1,0));

	// visual specs
	AddMesh ("longshot", &_V(0,0,0));

	SetCameraOffset (_V(-1.65,5.5,174));
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
}


// --------------------------------------------------------------
// Save/load vessele state to/from scenario
// --------------------------------------------------------------	
void LongShot::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{		//commented code  is for reference only

	char *line;
	while (oapiReadScenario_nextline (scn, line)) {

		// UMMU, Load all saved member from scenario.
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;

		// Lectura del canal del transpondedor
		else if (!strnicmp (line, "XPDR", 4))
		{
				sscanf (line+10, "%d%lf", &transpondedor);
				// Activar
				EnableTransponder(true);
				SetTransponderChannel(transpondedor);
		}

		//if (!strnicmp (line, "SOMEPARAM", 9)) {
		//	sscanf (line+9, "%lf", somevar);
		//} else {
			ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
		//}
	}
}

void LongShot::clbkSaveState (FILEHANDLE scn)
{	
	//commented code  is for reference only

	//char cbuf[256];
	// default vessel parameters
	VESSEL2::clbkSaveState (scn);

	// Save all UMMU member to scenario
	Crew.SaveAllMembersInOrbiterScenarios(scn);

	// custom parameters
	//sprintf (cbuf, "%0.1f", somefloatpara,);
	//oapiWriteScenario_string (scn, "SOMEPARAM", cbuf);
}

// --------------------------------------------------------------
// Called at each simulation time step before the state is updated
// --------------------------------------------------------------
void LongShot::clbkPreStep (double simt, double simdt, double mjd)
{
	//do something

	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
		case UMMU_TRANSFERED_TO_OUR_SHIP:
			sprintf(SendHudMessage(),"%s \"%s\" aged %i was transfered to our ship",
			Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
			,Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
		case UMMU_RETURNED_TO_OUR_SHIP:
			sprintf(SendHudMessage(),"%s \"%s\" aged %i entered into our ship",
			Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
			Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	}

	Crew.WarnUserUMMUNotInstalled("MyAddonName");

}

void LongShot::clbkPostStep (double simtt, double simdt, double mjd)
{

VECTOR3 velang;

	// Dar dato de velocidad angular
	GetAngularVel(velang);

	// Limitar velocidad angular apagando el grupo de motores correspondiente

	// Eje X
	if (velang.x > 0.05)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0);
		DelThrusterGroup(THGROUP_ATT_PITCHUP,FALSE);
	}
	else
	{
		th_group[0]=th_rcs[1];
		th_group[1]=th_rcs[6];
		th_group[2]=th_rcs[10];
		th_group[3]=th_rcs[15];
		CreateThrusterGroup(th_group,4,THGROUP_ATT_PITCHUP);
	}
	if (velang.x < -0.05)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0);
		DelThrusterGroup(THGROUP_ATT_PITCHDOWN,FALSE);
	}
	else
	{
		th_group[0]=th_rcs[0];
		th_group[1]=th_rcs[5];
		th_group[2]=th_rcs[11];
		th_group[3]=th_rcs[16];
		CreateThrusterGroup(th_group,4,THGROUP_ATT_PITCHDOWN);
	}

	// Eje Y
	if (velang.y > 0.05)
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0);
		DelThrusterGroup(THGROUP_ATT_YAWLEFT,FALSE);
	}
	else
	{
		th_group[0]=th_rcs[4];
		th_group[1]=th_rcs[19];
		CreateThrusterGroup(th_group,2,THGROUP_ATT_YAWLEFT);
	}
	if (velang.y < -0.05)
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0);
		DelThrusterGroup(THGROUP_ATT_YAWRIGHT,FALSE);
	}
	else
	{
		th_group[0]=th_rcs[9];
		th_group[1]=th_rcs[14];
		CreateThrusterGroup(th_group,2,THGROUP_ATT_YAWRIGHT);
	}

	// Eje Z
	if (velang.z > 0.15)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
		DelThrusterGroup(THGROUP_ATT_BANKRIGHT,FALSE);
	}
	else
	{
		th_group[0]=th_rcs[0];
		th_group[1]=th_rcs[6];
		th_group[2]=th_rcs[10];
		th_group[3]=th_rcs[16];
		CreateThrusterGroup(th_group,4,THGROUP_ATT_BANKRIGHT);
	}
	if (velang.z < -0.15)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
		DelThrusterGroup(THGROUP_ATT_BANKLEFT,FALSE);
	}
	else
	{
		th_group[0]=th_rcs[1];
		th_group[1]=th_rcs[5];
		th_group[2]=th_rcs[11];
		th_group[3]=th_rcs[15];
		CreateThrusterGroup(th_group,4,THGROUP_ATT_BANKLEFT);
	}

/*
	// Revisin del puerto de atraque 0 y mensaje de estado
	if (DockingStatus(0)==0)
		sprintf(SendHudMessage(),"Puerto 0 libre");
	else
	{
		oapiGetObjectName(GetDockStatus(GetDockHandle(0)),nombre,10);
		sprintf(SendHudMessage(),"Puerto 0 ocupado por %s",nombre);
	}

*/
}

void LongShot::CambioDePuerto()
{
	// Revisin del puerto de atraque y mensaje de estado
		if (DockingStatus(iActiveDockNumber)==0)
			strcpy(nombre,"None");
		else
			oapiGetObjectName(GetDockStatus(GetDockHandle(iActiveDockNumber)),nombre,10);
		sprintf(SendHudMessage(),"Active dock number changed to: %i %s",iActiveDockNumber,nombre);
		SetUMMUAirlockPos();
}

// --------------------------------------------------------------
//Keyboard state handler. Check OrbiterAPI reference for details
// --------------------------------------------------------------
int LongShot::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{

UINT i;

	// only process keydown events
	if (!down) 
		return 0;

	// SHIFT+1: Cambio de nivel de propulsores
	if (key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&KEYMOD_CONTROL(kstate))
	{
		if (MAXRCSTH==maximo)
		{
			MAXRCSTH=minimo;
			for (i=0; i<20; i++)
				SetThrusterMax0(th_rcs[i],MAXRCSTH);
			sprintf(SendHudMessage(),"RCS changed to minimum level");
		}
		else
		{
			MAXRCSTH=maximo;
			for (i=0; i<20; i++)
				SetThrusterMax0(th_rcs[i],MAXRCSTH);
			sprintf(SendHudMessage(),"RCS changed to maximum level");
		}
	}

	//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	//
	// ADD REALISM: It's your responsabilities also to set ship's control accordingly to crew aboard.
	//				If you want to disable control if no one is aboard have a look at "SetADCtrlMode()"
	//				and "SetAttitudeMode()" functions of Orbiter. To disable thrusters set their fuel 
	//				ressource to NULL.
	if(key==OAPI_KEY_E&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// PERFORM THE EVA, first we get is name with "GetCrewNameBySlotNumber" then we perform EVA with "EvaCrewMember"
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(iSelectedUmmuMember));
		// we provide feedback to user (You can display a message on panel or wathewer)
		// here below all the return code possible:
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendHudMessage(),"Transfer to docked ship Ok - %s transfered",
			Crew.GetLastEvaedCrewName()); iSelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendHudMessage(),"EVA OK - %s left the ship",
			Crew.GetLastEvaedCrewName()); iSelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(SendHudMessage(),"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(SendHudMessage(),"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendHudMessage(),"Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendHudMessage(),"Error, docked ship do not use UMmu 2.0, ask author to add it");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendHudMessage(),"Misc error with UMMU install it again");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	// you may do your own selection system by panel button, name etc etc
	if(key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(iSelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			iSelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(iSelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA",
		iSelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(iSelectedUmmuMember),
		Name,Crew.GetCrewAgeBySlotNumber(iSelectedUmmuMember));
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	// you may do your own selection system by panel button
	if(key==OAPI_KEY_2&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}
		if(iSelectedUmmuMember>0)
			iSelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(iSelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer"
			", please press \"E\" to EVA",iSelectedUmmuMember,
			Crew.GetCrewMiscIdBySlotNumber(iSelectedUmmuMember),Name,
			Crew.GetCrewAgeBySlotNumber(iSelectedUmmuMember));
		return 1;
	}
	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the virtual UMMU airlock door on/off
	if(key==OAPI_KEY_A&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// switch state
		Crew.SetAirlockDoorState(!Crew.GetAirlockDoorState());
		// display state
		if(Crew.GetAirlockDoorState()==TRUE)
			strcpy(SendHudMessage(),"Airlock is now open");	
		else
			strcpy(SendHudMessage(),"Airlock is now closed");	
		return 1;
	}
	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_S)
	{
		sprintf(SendHudMessage(),"%i souls aboard ship %s, %i seats available",
			Crew.GetCrewTotalNumber(),GetName(),50-Crew.GetCrewTotalNumber());
		return 1;
	}

	//---------------------------------------------------------------------------
	// Use a different Mesh (type "C" then EVA someone)
	// better idea is to use the new UMMU Id definition
	// look readme.txt in folder "config/UMMUIdConfig"
	if(key==OAPI_KEY_C)
	{
		Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy(SendHudMessage(),"Mesh changed");
		return 1;
	}

	// change active dock
	if(key==OAPI_KEY_SUBTRACT&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(iActiveDockNumber > 0)
		iActiveDockNumber--;
		CambioDePuerto();
		return 1;
	}
	//---------------------------------------------------------------------------
	// change active dock
	if(key==OAPI_KEY_ADD&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(iActiveDockNumber < 6)
		iActiveDockNumber++;
		CambioDePuerto();
		return 1;
	}

	return 0;
}


// --------------------------------------------------------------
// Orbiter's HUD callback
// used to display UMMU's message see PDF doc:
// "Example of feedback method by HUD Display"
// --------------------------------------------------------------
void LongShot::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	// UMmu display messages
	if(dHudMessageDelay>0)
	{
		TextOut (hDC,5,hps->H/60*15,cUmmuHudDisplay,strlen(cUmmuHudDisplay));
		dHudMessageDelay-=oapiGetSimStep();
		if(dHudMessageDelay<0)
			dHudMessageDelay=0;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////
// SetUMMUAirlockPos
///////////////////////////////////////////////////////////////////////////////////////////
void LongShot::SetUMMUAirlockPos(void)
{

	int AirlockStatus=Crew.GetAirlockDoorState();
	Crew.SetActiveDockForTransfer(iActiveDockNumber);
	
	switch(iActiveDockNumber)
	{
	case 0:	

		Crew.DefineAirLockShape(AirlockStatus,-15,15,-15,15,194,224);			
		Crew.SetMembersPosRotOnEVA(_V(0,0,209),_V(0,0,0));	
		//optional define here camera that show this dock
		break;
	case 1:	

		Crew.DefineAirLockShape(TRUE,-15,15,-15,15,-15,15);			
		Crew.SetMembersPosRotOnEVA(_V(5.5,0,-4.67),_V(0,0,0));	
		//optional define here camera that show this dock
		break;
	case 2:	

		Crew.DefineAirLockShape(TRUE,-15,15,-15,15,-15,15);			
		Crew.SetMembersPosRotOnEVA(_V(-5.5,0,-4.67),_V(0,0,0));	
		//optional define here camera that show this dock
		break;
	case 3:	

		Crew.DefineAirLockShape(TRUE,5,35,-15,15,32,62);			
		Crew.SetMembersPosRotOnEVA(_V(20,0,47),_V(0,0,0));	
		//optional define here camera that show this dock
		break;
	case 4:	

		Crew.DefineAirLockShape(TRUE,-35,-5,-15,15,32,62);			
		Crew.SetMembersPosRotOnEVA(_V(-20,0,47),_V(0,0,0));	
		//optional define here camera that show this dock
		break;
	case 5:	

		Crew.DefineAirLockShape(TRUE,-15,15,-35,-5,32,62);			
		Crew.SetMembersPosRotOnEVA(_V(0,-20,47),_V(0,0,0));	
		//optional define here camera that show this dock
		break;
	case 6:	

		Crew.DefineAirLockShape(TRUE,-15,15,5,35,32,62);			
		Crew.SetMembersPosRotOnEVA(_V(0,20,47),_V(0,0,0));	
		//optional define here camera that show this dock
		break;
	}
}
/*
	Crew.DefineAirLockShape(FALSE,-3,3,-4,4,-7,7);// Airlock closed, 6 meter large 8 meter high 14 meter long
	CreateDock (_V(0,0,207.2), _V(0,0,1), _V(0,1,0));
	CreateDock (_V(4.57,0,-4.67), _V(1,0,0), _V(0,0,-1));
	CreateDock (_V(-4.57,0,-4.67), _V(-1,0,0), _V(0,0,-1));
	CreateDock (_V(20,0,49), _V(0,0,-1), _V(1,0,0));
	CreateDock (_V(-20,0,49), _V(0,0,-1), _V(-1,0,0));
	CreateDock (_V(0,-20,49), _V(0,0,-1), _V(0,-1,0));
	CreateDock (_V(0,20,49), _V(0,0,-1), _V(0,1,0));
*/

// ==============================================================
// API callback interface
// ==============================================================
// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new LongShot (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (LongShot*)vessel;
}
